/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
import type {Config} from '@jest/types';
import type {FS} from 'jest-haste-map';
import type {MatcherState} from 'expect';
import type {OptionsReceived} from 'pretty-format';
import {Plugin as Plugin_2} from 'pretty-format';
import {Plugins} from 'pretty-format';

export declare const addSerializer: (plugin: Plugin_2) => void;

export declare const buildSnapshotResolver: (
  config: Config.ProjectConfig,
  localRequire?: Promise<LocalRequire> | LocalRequire,
) => Promise<SnapshotResolver>;

export declare const cleanup: (
  hasteFS: FS,
  update: Config.SnapshotUpdateState,
  snapshotResolver: SnapshotResolver,
  testPathIgnorePatterns?: string[] | undefined,
) => {
  filesRemoved: number;
  filesRemovedList: Array<string>;
};

declare type Context = MatcherState & {
  snapshotState: SnapshotState;
};

declare type ExpectationResult = {
  pass: boolean;
  message: () => string;
};

export declare const EXTENSION = 'snap';

export declare const getSerializers: () => Plugins;

export declare const isSnapshotPath: (path: string) => boolean;

declare type LocalRequire = (module: string) => unknown;

declare type SaveStatus = {
  deleted: boolean;
  saved: boolean;
};

declare type SnapshotMatchOptions = {
  testName: string;
  received: unknown;
  key?: string;
  inlineSnapshot?: string;
  isInline: boolean;
  error?: Error;
};

export declare type SnapshotResolver = {
  testPathForConsistencyCheck: string;
  resolveSnapshotPath(testPath: Config.Path, extension?: string): Config.Path;
  resolveTestPath(snapshotPath: Config.Path, extension?: string): Config.Path;
};

declare type SnapshotReturnOptions = {
  actual: string;
  count: number;
  expected?: string;
  key: string;
  pass: boolean;
};

export declare class SnapshotState {
  private _counters;
  private _dirty;
  private _index;
  private _updateSnapshot;
  private _snapshotData;
  private _initialData;
  private _snapshotPath;
  private _inlineSnapshots;
  private _uncheckedKeys;
  private _prettierPath;
  private _snapshotFormat;
  added: number;
  expand: boolean;
  matched: number;
  unmatched: number;
  updated: number;
  constructor(snapshotPath: Config.Path, options: SnapshotStateOptions);
  markSnapshotsAsCheckedForTest(testName: string): void;
  private _addSnapshot;
  clear(): void;
  save(): SaveStatus;
  getUncheckedCount(): number;
  getUncheckedKeys(): Array<string>;
  removeUncheckedKeys(): void;
  match({
    testName,
    received,
    key,
    inlineSnapshot,
    isInline,
    error,
  }: SnapshotMatchOptions): SnapshotReturnOptions;
  fail(testName: string, _received: unknown, key?: string): string;
}

declare type SnapshotStateOptions = {
  updateSnapshot: Config.SnapshotUpdateState;
  prettierPath: Config.Path;
  expand?: boolean;
  snapshotFormat: OptionsReceived;
};

export declare const toMatchInlineSnapshot: (
  this: Context,
  received: unknown,
  propertiesOrSnapshot?: string | object | undefined,
  inlineSnapshot?: string | undefined,
) => ExpectationResult;

export declare const toMatchSnapshot: (
  this: Context,
  received: unknown,
  propertiesOrHint?: string | object | undefined,
  hint?: string | undefined,
) => ExpectationResult;

export declare const toThrowErrorMatchingInlineSnapshot: (
  this: Context,
  received: unknown,
  inlineSnapshot?: string | undefined,
  fromPromise?: boolean | undefined,
) => ExpectationResult;

export declare const toThrowErrorMatchingSnapshot: (
  this: Context,
  received: unknown,
  hint: string | undefined,
  fromPromise: boolean,
) => ExpectationResult;

export {};
